package org.foxarmy.barcodescannerforemployees

import android.content.Context
import android.os.Build
import androidx.annotation.RequiresApi
import okhttp3.*
import org.foxarmy.barcodescannerforemployees.database.AbstractProductDAO
import org.foxarmy.barcodescannerforemployees.database.CategoryDAO
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.database.ProductDAO
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import org.foxarmy.barcodescannerforemployees.dataclasses.Category
import org.foxarmy.barcodescannerforemployees.dataclasses.Product
import org.json.JSONObject

class WebSocketClient(private val context: Context, private val server: String) {

    private lateinit var webSocket: WebSocket

    fun connect(token: String, currentGroup: String) {
        val client = OkHttpClient()
        val request = Request.Builder().url("ws://$server:8282").build()
        client.newWebSocket(request, object : WebSocketListener() {
            override fun onOpen(webSocket: WebSocket, response: Response) {
                this@WebSocketClient.webSocket = webSocket
                sendMessage("\"token\":\"$token\",\"currentGroup\":\"$currentGroup\"")
            }

            @RequiresApi(Build.VERSION_CODES.O)
            override fun onMessage(webSocket: WebSocket, text: String) {
                val payload = JSONObject(text)

                val item = payload["item"] as String
                val data = payload["data"] as JSONObject
                val action = payload["action"] as String
                val currentGroup = payload["groupId"] as String

                val dbHelper = DBStorageController(context, currentGroup)
                val abstractProductDAO = AbstractProductDAO(dbHelper)
                val productDAO = ProductDAO(dbHelper)
                val categoryDAO = CategoryDAO(dbHelper)

                when (action) {
                    "create" -> {
                        when (item) {
                            "abstractproduct" -> {
                                val newAbstractProduct = AbstractProduct.createFromJSON(data)
                                abstractProductDAO.addAbstractProduct(newAbstractProduct)
                            }

                            "product" -> {
                                val newProduct = Product.createFromJSON(data)
                                productDAO.insertNewProduct(newProduct)
                            }

                            "category" -> {
                                val newCategory = Category.createFromJSON(data)
                                categoryDAO.addCategory(newCategory)
                            }
                        }
                    }

                    "update" -> {
                        when (item) {
                            "abstractproduct" -> {
                                val updatedAbstractProduct = AbstractProduct.createFromJSON(data)
                                abstractProductDAO.updateAbstractProduct(updatedAbstractProduct)
                            }
                            "product" -> {
                                val updatedProduct = Product.createFromJSON(data)
                                productDAO.updateProduct(updatedProduct)
                            }
                            "category" -> {
                                val updatedCategory = Category.createFromJSON(data)
                                categoryDAO.updateCategory(updatedCategory)
                            }
                        }
                    }

                    "delete" -> {
                        val id = data["local_id"] as Int

                        when(item) {
                            "abstractproduct" -> {
                                abstractProductDAO.eraseAbstractProduct(id, context)
                            }
                            "product" -> {
                                productDAO.eraseProduct(id)
                            }
                            "category" -> {
                                categoryDAO.eraseCategory(id, context)
                            }
                        }
                    }
                }
            }

            override fun onClosing(webSocket: WebSocket, code: Int, reason: String) {
            }

            override fun onFailure(webSocket: WebSocket, t: Throwable, response: Response?) {
            }
        })
    }

    fun sendMessage(message: String) {
        webSocket.send(message)
    }

    fun close() {
        webSocket.close(1000, "Closing WebSocket connection")
    }
}