package org.foxarmy.barcodescannerforemployees.activities

import android.content.Intent
import android.os.Bundle
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKeys
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.databinding.ActivityGroupBinding
import org.foxarmy.barcodescannerforemployees.noInternetConnectionAvailableNotification

class GroupActivity : AppCompatActivity() {

    private lateinit var binding: ActivityGroupBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityGroupBinding.inflate(layoutInflater)
        setContentView(binding.root)

        val sharedPreferences = EncryptedSharedPreferences.create(
            "sensitive",
            MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC),
            applicationContext,
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        binding.createGroupButton.setOnClickListener {
            val groupName = binding.groupNameTextEdit.text.toString()
            val groupPassword = binding.groupPasswordTextEdit.text.toString()
            val net = Net()
            net.language = sharedPreferences.getString("language", "en-US")!!
            net.server = sharedPreferences.getString("server", "")!!
            net.token = sharedPreferences.getString("token", "")!!

            if (!net.serverIsAvailable(this)) {
                noInternetConnectionAvailableNotification(this)
            } else {
                val response = net.createGroup(groupName, groupPassword)
                val responseText = response.body!!.string()

                if (response.code == 200) {
                    val currentGroups = sharedPreferences.getStringSet("groups", mutableSetOf())
                    currentGroups!!.add(responseText)
                    sharedPreferences.edit().putStringSet("groups", currentGroups).apply()
                    sharedPreferences.edit().putString("currentGroup", responseText).apply()
                    val intent = Intent(this, MainActivity::class.java)
                    startActivity(intent)
                    finish()
                } else {
                    Toast.makeText(this, responseText, Toast.LENGTH_LONG).show()
                }
            }
        }

        binding.joinGroupButton.setOnClickListener {
            val groupName = binding.groupNameTextEdit.text.toString()
            val groupPassword = binding.groupPasswordTextEdit.text.toString()

            val net = Net()
            net.language = sharedPreferences.getString("language", "en-US")!!
            net.server = sharedPreferences.getString("server", "")!!
            net.token = sharedPreferences.getString("token", "")!!

            if (!net.serverIsAvailable(this)) {
                noInternetConnectionAvailableNotification(this)
            } else {
                val requestGroupIdResponse = net.getGroupId(groupName)
                var groupId: Int
                try {
                   groupId = requestGroupIdResponse.toInt()
                } catch (e: Exception) {
                    Toast.makeText(this, requestGroupIdResponse, Toast.LENGTH_SHORT).show()
                    return@setOnClickListener
                }
                val response = net.joinGroup(groupId, groupPassword)
                val responseText = response.body!!.string()

                if (response.code == 200) {
                    val currentGroups = sharedPreferences.getStringSet("groups", mutableSetOf())
                    currentGroups!!.add(groupId.toString())
                    sharedPreferences.edit().putStringSet("groups", currentGroups).apply()
                    sharedPreferences.edit().putString("currentGroup", groupId.toString()).apply()
                    val intent = Intent(this, MainActivity::class.java)
                    startActivity(intent)
                    finish()
                } else {
                    Toast.makeText(this, responseText, Toast.LENGTH_LONG).show()
                }
            }
        }
    }
}